import 'package:analyzer/dart/analysis/analysis_context_collection.dart';
import 'package:analyzer/dart/analysis/results.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/visitor.dart';
import 'package:analyzer/file_system/overlay_file_system.dart';
import 'package:analyzer/file_system/physical_file_system.dart';

void main() async {
  final provider = OverlayResourceProvider(PhysicalResourceProvider.INSTANCE);
  provider.setOverlay(
    '/foo/bar.dart',
    content: '''
class Property {
  final String? name;

  const Property([this.name]);
}

void x() {
  @Property() int a, b;
}
''',
    modificationStamp: 0,
  );

  final collection = AnalysisContextCollection(
      includedPaths: ['/foo'], resourceProvider: provider);
  final file = await collection.contexts.single.currentSession
      .getResolvedLibrary('/foo/bar.dart') as ResolvedLibraryResult;

  file.units.single.unit.accept(_Visitor());
}

class _Visitor extends RecursiveAstVisitor<void> {
  @override
  void visitVariableDeclaration(VariableDeclaration node) {
    final element = node.declaredElement;
    if (element != null) {
      print('${element.name}: ${element.metadata}');
    }
  }
}
